//+------------------------------------------------------------------+
//|                                                  Credentials.mqh |
//|                        Copyright 2022, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include "Program.mqh"
//+------------------------------------------------------------------+
//| Credentials                                                      |
//+------------------------------------------------------------------+
struct Credentials {
   int      id;
   string   param;
   string   value;
};
//+------------------------------------------------------------------+
//| Creating a Credentials table                                     |
//+------------------------------------------------------------------+
bool CProgram::CreateTableCredentials(const int db) {
   ResetLastError();
   if(!DatabaseTableExists(db, m_credentials_table)) {
      if(!DatabaseExecute(db, "CREATE TABLE "+m_credentials_table+"("
                          "ID    INT  PRIMARY KEY NOT NULL,"
                          "PARAM TEXT NOT NULL,"
                          "VALUE TEXT);"))
      {
         Print("Create table failed with code ", GetLastError());
         DatabaseClose(db);
         return false;
      }
   }
   return true;
}
//+------------------------------------------------------------------+
//| Insert credentials                                               |
//+------------------------------------------------------------------+
bool CProgram::InsertCredentials(const int db) {
   int res = MessageBox(
      "Save credentials for the next login?", 
      "Saving credentials", MB_YESNO
   );
   if(res == IDYES) {
      ResetLastError();
      if(!DatabaseExecute(db, 
         "INSERT INTO "+m_credentials_table+" (ID,PARAM,VALUE) VALUES (1,'email','"+m_email_edit.GetValue()+"'); "
         "INSERT INTO "+m_credentials_table+" (ID,PARAM,VALUE) VALUES (2,'password','"+m_password_edit.GetValue()+"');"
      )) {
         Print("Insert failed with code ", GetLastError());
         DatabaseClose(db);
         return false;
      }
   }
   return true;
}
//+------------------------------------------------------------------+
//| Update credentials                                               |
//+------------------------------------------------------------------+
bool CProgram::UpdateCredentials(const int db) {
   int res =MessageBox(
      "Save credentials for the next login?", 
      "Saving data", MB_YESNO
   );
   if(res == IDYES) {
      ResetLastError();
      if(!DatabaseExecute(db, 
         "UPDATE "+m_credentials_table+" SET value = '"+m_email_edit.GetValue()+"' WHERE param = 'email';"
         "UPDATE "+m_credentials_table+" SET value = '"+m_password_edit.GetValue()+"' WHERE param = 'password';"
      )) {
         Print("Update failed with code ", GetLastError());
         DatabaseClose(db);
         return false;
      }
   }
   return true;
}
//+------------------------------------------------------------------+
//| Get credentials                                                  |
//+------------------------------------------------------------------+
CJAVal CProgram::GetCredentials(void) {
   CJAVal data;
   data["email"]    ="";
   data["password"] ="";
   
   int db = -1;
   if((db = CProgram::OpenDB(m_db_name)) < 0) {
      return false;
   }
   
   ResetLastError();
   if(DatabaseTableExists(db, m_credentials_table)) {
      int request = DatabasePrepare(db, 
         "SELECT * FROM "+m_credentials_table+" WHERE PARAM = 'email' OR PARAM = 'password'"
      );
      if(request == INVALID_HANDLE) {
         Print("Request failed with code ", GetLastError());
         DatabaseClose(db);
         return false;
      }
      Credentials credentials;
      for(int i=0; DatabaseReadBind(request, credentials); i++) {
         if(credentials.param == "email") {
            data["email"] = credentials.value;
         }
         else if(credentials.param == "password") {
            data["password"] = credentials.value;
         }
      }
      DatabaseFinalize(request);
   }
   DatabaseClose(db);
   return data;
}
//+------------------------------------------------------------------+
//| Save credentials                                                 |
//+------------------------------------------------------------------+
void CProgram::SaveCredentials(void) {
   int db = -1;
   if((db = CProgram::OpenDB(m_db_name)) < 0) {
      return;
   }
   
   if(!CProgram::CreateTableCredentials(db)) {
      return;
   }
   
   long line_total = 0;
   if((line_total = CProgram::NumberOfLinesRecieved(db, 
         "SELECT * FROM "+m_credentials_table+" WHERE PARAM = 'email' OR PARAM = 'password'")) < 0) {
      return;
   }
   if(!line_total) {
      if(!CProgram::InsertCredentials(db)) {
         return;
      }
   } else {
      if(!CProgram::UpdatingCredentialsIfChanged(db)) {
         return;
      }
   }
   DatabaseClose(db);
}
//+------------------------------------------------------------------+
//| Delete credentials                                               |
//+------------------------------------------------------------------+
bool CProgram::DeleteCredentials(void) {
   int db = -1;
   if((db = CProgram::OpenDB(m_db_name)) < 0) {
      return false;
   }
   int res = MessageBox(
      "Do you really want to delete your credentials?\nAfter you delete them, you will have to enter them yourself.", 
      "Deleting credentials", MB_YESNO
   );
   if(res == IDYES) {
      ResetLastError();
      if(!DatabaseExecute(db, 
         "UPDATE "+m_credentials_table+" SET value = '' WHERE param = 'email';"
         "UPDATE "+m_credentials_table+" SET value = '' WHERE param = 'password';"
      )) {
         Print("Delete failed with code ", GetLastError());
         DatabaseClose(db);
         return false;
      }
      m_email_edit.SetValue("");
      m_password_edit.SetValue("");
      CWndEvents::Update(true);
   }
   DatabaseClose(db);
   return true;
}
//+------------------------------------------------------------------+
//| Updating credentials if changed                                  |
//+------------------------------------------------------------------+
bool CProgram::UpdatingCredentialsIfChanged(const int db) {
   int request = DatabasePrepare(db, 
      "SELECT * FROM "+m_credentials_table+" WHERE PARAM = 'email' OR PARAM = 'password'"
   );
   if(request == INVALID_HANDLE) {
      Print("Request failed with code ", GetLastError());
      DatabaseClose(db);
      return false;
   }
   bool is_changed = false;
   Credentials cr[];
   Credentials credentials;
   for(int i=0; DatabaseReadBind(request, credentials); i++) {
      int size = ArraySize(cr);
      ArrayResize(cr, size+1);
      cr[size] = credentials;
      if((credentials.param == "email" && credentials.value != m_email_edit.GetValue()) ||
         (credentials.param == "password" && credentials.value != m_password_edit.GetValue())) {
         is_changed = true;
      }
   }
   ArrayPrint(cr);
   DatabaseFinalize(request);
   
   if(is_changed) {
      CProgram::UpdateCredentials(db);
   }
   return true;
}
//+------------------------------------------------------------------+
