//+------------------------------------------------------------------+
//|                                                      Request.mqh |
//|                        Copyright 2022, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include "Program.mqh"
//+------------------------------------------------------------------+
//| Defines                                                          |
//+------------------------------------------------------------------+
#define BASE_URL     "http://localhost:8000"
#define BASE_API_URL "http://localhost:8000/api"
//+------------------------------------------------------------------+
//| Request                                                          |
//+------------------------------------------------------------------+
int CProgram::Request(string method, string url, 
                        string request_headers, int timeout, char &request_data[],
                        char &response_body[], string &response_headers) 
{
   ResetLastError();
   int response_code = 
      WebRequest(method, url, 
         request_headers, timeout, request_data, 
         response_body, response_headers);
   
   int error =GetLastError();
   Print("\nresponse_code: ", response_code, ", error: ", error);

   if(response_code < 0 || error > 0) {
      CProgram::PrintResponse(response_headers, response_body, response_code);
      return -1;
   }
   
   return response_code;
}
//+------------------------------------------------------------------+
//| Token request in cookies                                         |
//+------------------------------------------------------------------+
bool CProgram::RequestCsrfCookie(void) {
   string url = BASE_API_URL + "/sanctum/csrf-cookie";
   char   request_data[];
   char   response_body[];
   string response_headers ="";

   if((m_last_response_code = this.Request("GET", url, 
         "", NULL, request_data, 
         response_body, response_headers)) < 0) {
      return false;
   }
   
   CProgram::SaveXsrfData(response_headers);
   CProgram::PrintResponse(response_headers, response_body, m_last_response_code);
   return true;
}
//+------------------------------------------------------------------+
//| User authorization request                                       |
//+------------------------------------------------------------------+
bool CProgram::RequestLogin(void) {
   if(m_email_edit.GetValue() == "" || m_password_edit.GetValue() == "") {
      int res = MessageBox(
         "Email or Password is empty!", 
         "Error"
      );
      return false;
   }
   
   if(!CProgram::RequestCsrfCookie()) {
      return false;
   }
   
   string url = BASE_API_URL + "/login";
   string request_headers = CProgram::GetHeaders();
   string request_params = 
      "email=" + m_email_edit.GetValue() + 
      "&password=" + m_password_edit.GetValue();
   char   request_data[];
   char   response_body[];
   string response_headers;
   
   Print("[request_headers]");
   Print(request_headers);
   
   ArrayResize(request_data, 
      StringToCharArray(request_params,request_data,0,WHOLE_ARRAY,CP_UTF8)-1
   );
   
   if((m_last_response_code = this.Request("POST", url,
         request_headers, 10000, request_data, 
         response_body, response_headers)) < 0) {
      return false;
   }
   CProgram::SaveXsrfData(response_headers);
   CProgram::PrintResponse(response_headers, response_body, m_last_response_code);
   
   m_status_bar.SetValue(1, "Connected");
   m_status_bar.GetItemPointer(1).Update(true);
   
   CProgram::SaveCredentials();
   return true;
}
//+------------------------------------------------------------------+
//| Request to retrieve user data                                    |
//+------------------------------------------------------------------+
bool CProgram::RequestUserData(void) {
   string url = BASE_API_URL + "/user";
   string request_headers = CProgram::GetHeaders();
   char   request_data[];
   char   response_body[];
   string response_headers;
   
   Print("[request_headers]");
   Print(request_headers);
   
   if((m_last_response_code = this.Request("GET", url, 
         request_headers, NULL, request_data, 
         response_body, response_headers)) < 0) {
      return false;
   }
   CProgram::SaveXsrfData(response_headers);
   CProgram::PrintResponse(response_headers, response_body, m_last_response_code);
   
   if(m_last_response_code == 200) {
      CJAVal data;
      data.Deserialize(CharArrayToString(response_body));
      string user_data = "";
      user_data += "id:\t\t" + data["id"].ToStr();
      user_data += "\nname:\t\t" + data["name"].ToStr();
      user_data += "\nemail:\t\t" + data["email"].ToStr();
      user_data += "\nemail_verified_at:\t" + 
         ((data["email_verified_at"].ToStr() == "")? 
            "-" : 
            string(StringToTime(data["email_verified_at"].ToStr())));
      user_data += "\ncreated_at:\t" + string(StringToTime(data["created_at"].ToStr()));
      user_data += "\nupdated_at:\t" + string(StringToTime(data["updated_at"].ToStr()));
      
      int res = MessageBox(
         user_data, "User data"
      );
   }
   return true;
}
//+------------------------------------------------------------------+
//| Logout request                                                   |
//+------------------------------------------------------------------+
bool CProgram::RequestLogout(void) {
   string url = BASE_API_URL + "/logout";
   string request_headers = CProgram::GetHeaders();
   uchar  request_data[];
   uchar  response_body[];
   string response_headers;
   
   Print("[request_headers]");
   Print(request_headers);
   
   if((m_last_response_code = this.Request("GET", url, 
         request_headers, NULL, request_data, 
         response_body, response_headers)) < 0) {
      return false;
   }
   m_xsrf_token = "";
   
   CProgram::PrintResponse(response_headers, response_body, m_last_response_code);
   
   m_status_bar.SetValue(1, "Disconnected");
   m_status_bar.GetItemPointer(1).Update(true);
   return true;
}
//+------------------------------------------------------------------+
//| New user registration request                                    |
//+------------------------------------------------------------------+
bool CProgram::RequestRegister(void) {
   string url = BASE_API_URL + "/register";
   string request_headers = CProgram::GetHeaders();
   string request_params = 
      "name=" + m_signup_name_edit.GetValue() + 
      "&email=" + m_signup_email_edit.GetValue() + 
      "&password=" + m_signup_password_edit.GetValue();
   char   request_data[];
   char   response_body[];
   string response_headers;
   
   Print("[request_headers]");
   Print(request_headers);
   
   ArrayResize(request_data, 
      StringToCharArray(request_params,request_data,0,WHOLE_ARRAY,CP_UTF8)-1
   );
   
   if((m_last_response_code = this.Request("POST", url,
         request_headers, 10000, request_data, 
         response_body, response_headers)) < 0) {
      return false;
   }
   CProgram::SaveXsrfData(response_headers);
   CProgram::PrintResponse(response_headers, response_body, m_last_response_code);
   return true;
}
//+------------------------------------------------------------------+
//| Logs the server response                                         |
//+------------------------------------------------------------------+
void CProgram::PrintResponse(string response_headers, 
                              char &response_body[], 
                              int response_code, 
                              int line_total = 10) {
   Print("\n[response_headers]");
   Print(response_headers);
   
   Print("[response_body]");
   string body = CharArrayToString(response_body);
   string sbody[];
   string sep="\n";
   ushort u_sep;
   u_sep=StringGetCharacter(sep, 0);
   StringSplit(body, u_sep, sbody);
   
   Print("string length: ", StringLen(body));
   Print("array size: ", ArraySize(sbody));
   
   if(response_code >= 200 && response_code < 300) {
      Print(body);
      return;
   }
   if(response_code >= 300 && response_code < 600) {
      for(int i=0; i<ArraySize(sbody); i++) {
         Print(sbody[i]);
         if(i > line_total) {
            Print("...");
            break;
         }
      }
   }
}
//+------------------------------------------------------------------+
//| Forms headers                                                    |
//+------------------------------------------------------------------+
string CProgram::GetHeaders(void) {
   string eol = "\r\n";
   string headers = "";
   headers += (m_xsrf_token != ""? "X-XSRF-TOKEN: " + m_xsrf_token + eol : "");
   headers += "Accept: application/json" + eol;
   headers += "Referer: localhost:3000";
   return headers;
}
//+------------------------------------------------------------------+
//| Saves received XSRF-data                                         |
//+------------------------------------------------------------------+
void CProgram::SaveXsrfData(string response_headers) {
   CRegex *rgx = new CRegex("(Set-Cookie:\\s*XSRF-TOKEN=)(?<token>.+?)(%3D;)");
   CMatch *m=rgx.Match(response_headers);
   if(m.Success()) {
      m_xsrf_token = m.Groups()["token"].Value();
   }
   delete m;
   delete rgx;
}
//+------------------------------------------------------------------+
