export default {
  // Global page headers: https://go.nuxtjs.dev/config-head
  head: {
    title: 'mt5',
    htmlAttrs: {
      lang: 'en'
    },
    meta: [
      { charset: 'utf-8' },
      { name: 'viewport', content: 'width=device-width, initial-scale=1' },
      { hid: 'description', name: 'description', content: '' }
    ],
    link: [
      { rel: 'icon', type: 'image/x-icon', href: '/favicon.ico' }
    ]
  },

  // Global CSS: https://go.nuxtjs.dev/config-css
  css: [
    '~/assets/scss/app.scss'
  ],

  // Plugins to run before rendering page: https://go.nuxtjs.dev/config-plugins
  plugins: [
    { src: '~/plugins/ui/iview.js', ssr: true },
  ],

  // Auto import components: https://go.nuxtjs.dev/config-components
  components: true,

  // Modules for dev and build (recommended): https://go.nuxtjs.dev/config-modules
  buildModules: [],

  // Modules: https://go.nuxtjs.dev/config-modules
  modules: [
    '@nuxtjs/dotenv',
    '@nuxtjs/axios',
    '@nuxtjs/proxy',
    '@nuxtjs/auth-next',
  ],

  axios: {
    credentials: true
  },

  proxy: {
    "/api": {
      target: process.env.API_DOMAIN,
      pathRewrite: { "^/api": "/" }
    }
  },

  auth: {
    redirect: {
      login: '/auth/login',
      logout: '/',
      home: '/',
    },
    strategies: {
      laravelSanctum: {
        provider: "laravel/sanctum",
        url: process.env.API_URL,
        endpoints: {
          login: { url: '/login' },
          user: { url: '/user' },
          logout: { url: '/logout', method: 'get' },
        },
      },
    }
  },

  // Build Configuration: https://go.nuxtjs.dev/config-build
  build: {}
}
